package ab.table;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import ab.Address;
import ab.Category;

/**
 * The model for the AddressTable.
 * 
 * @author ahoefer
 * 
 */
public class AddressTableModel extends AbstractTableModel {

	private static final long serialVersionUID = -7996032405054969091L;

	private String[] columnNames = { "Kategorie", "Vorname", "Nachname",
			"Adresse 1", "Addresse 2", "PLZ", "Stadt", "Land", "Festnetz",
			"Mobil", "E-Mail" };

	private List<Address> addresses = new ArrayList<Address>();

	/**
	 * Gets the column name.
	 * 
	 * @param column
	 *            the column's index
	 * @return the column name
	 */
	@Override
	public String getColumnName(int column) {
		return columnNames[column];
	}

	/**
	 * Gets the number of columns.
	 * 
	 * @return The number of columns.
	 * @see AbstractTableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return columnNames.length;
	}

	/**
	 * Gets the number of rows.
	 * 
	 * @return The number of rows.
	 * @see AbstractTableModel#getRowCount()
	 */
	public int getRowCount() {
		return addresses.size();
	}

	/**
	 * Gets the value for the cell at the given row and column.
	 * 
	 * @param row
	 *            The cell's row.
	 * @param column
	 *            The cell's column.
	 * @return The value of the specified cell.
	 * @see AbstractTableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int row, int column) {
		Address a = addresses.get(row);
		switch (column) {
		case 0:
			return a.getCategory();
		case 1:
			return a.getFirstname();
		case 2:
			return a.getLastname();
		case 3:
			return a.getAddress1();
		case 4:
			return a.getAddress2();
		case 5:
			return a.getZipCode();
		case 6:
			return a.getCity();
		case 7:
			return a.getCounty();
		case 8:
			return a.getLandline();
		case 9:
			return a.getMobile();
		case 10:
			return a.getEMail();
		default:
			throw new IndexOutOfBoundsException("Column " + column
					+ "does not exist.");
		}
	}

	/**
	 * Gets the column's class
	 * 
	 * @param column
	 *            the column's index
	 * @return the column's class
	 */
	@Override
	public Class<?> getColumnClass(int column) {
		if (column == 0) {
			return Category.class;
		}
		return String.class;
	}

	/**
	 * Gets a copy of the address at the specified index.
	 * 
	 * @param index
	 *            The index of the address.
	 * @return A copy of the address at the specified index.
	 */
	public Address getAddress(int index) {
		return new Address(addresses.get(index));
	}

	/**
	 * Adds a copy of the address to the model and notifies the table.
	 * 
	 * @param address
	 *            The address to add. May not be null.
	 */
	public void addAddress(Address address) {
		addresses.add(new Address(address));
		int row = addresses.size() - 1;
		fireTableRowsInserted(row, row);
	}

	/**
	 * Copies the content from the specified address to the address at the
	 * specified index.
	 * 
	 * @param index
	 *            The index of the address.
	 * @param address
	 *            The address from which the content shall be copied. May not be
	 *            null.
	 */
	public void updateAddress(int index, Address address) {
		addresses.get(index).copyAll(address);
		fireTableRowsUpdated(index, index);
	}

	/**
	 * Removes the address at the specified index and notifies the table.
	 * 
	 * @param index
	 *            The index of the address.
	 */
	public void removeAddress(int index) {
		addresses.remove(index);
		fireTableRowsDeleted(index, index);
	}
}
